@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2007 Sun Microsystems Inc.
+# Copyright 2004-2008 Sun Microsystems Inc.
 # Copyright 2001, 2002 BAUM Retec, A.G.
 #
 # This library is free software; you can redistribute it and/or
@@ -15,22 +15,25 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 
 """Provides methods that convert the role name of an Accessible
 object into localized strings for speech and braille."""
 
-__id__        = "$Id: rolenames.py 2361 2007-05-09 01:20:06Z lmonsanto $"
-__version__   = "$Revision: 2361 $"
-__date__      = "$Date: 2007-05-08 21:20:06 -0400 (Tue, 08 May 2007) $"
-__copyright__ = "Copyright (c) 2004-2007 Sun Microsystems Inc."
+__id__        = "$Id: rolenames.py 4485 2009-01-31 15:43:29Z wwalker $"
+__version__   = "$Revision: 4485 $"
+__date__      = "$Date: 2009-01-31 10:43:29 -0500 (Sat, 31 Jan 2009) $"
+__copyright__ = "Copyright (c) 2004-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import debug
 import settings
 
-from orca_i18n import _ # for gettext support
+import pyatspi
+
+from orca_i18n import _  # for gettext support
+from orca_i18n import Q_ # to provide qualified translatable strings
 
 ########################################################################
 #                                                                      #
@@ -38,6 +41,9 @@ from orca_i18n import _ # for gettext support
 #                                                                      #
 ########################################################################
 
+#[[[TODO: eitani - These are here for backward compatability, they should 
+#disappear]]]
+
 ROLE_INVALID             = "invalid"
 ROLE_ACCEL_LABEL         = "accelerator label"
 ROLE_ALERT               = "alert"
@@ -48,7 +54,6 @@ ROLE_CAPTION             = "caption"
 ROLE_CANVAS              = "canvas"
 ROLE_CHECK_BOX           = "check box"
 ROLE_CHECK_MENU_ITEM     = "check menu item"
-ROLE_CHECK_MENU          = "check menu" # invented for items that are submenus
 ROLE_COLOR_CHOOSER       = "color chooser"
 ROLE_COLUMN_HEADER       = "column header"
 ROLE_COMBO_BOX           = "combo box"
@@ -91,7 +96,6 @@ ROLE_PROGRESS_BAR        = "progress bar"
 ROLE_PUSH_BUTTON         = "push button"
 ROLE_RADIO_BUTTON        = "radio button"
 ROLE_RADIO_MENU_ITEM     = "radio menu item"
-ROLE_RADIO_MENU          = "radio menu" # invented for items that are submenus
 ROLE_ROOT_PANE           = "root pane"
 ROLE_ROW_HEADER          = "row header"
 ROLE_SCROLL_BAR          = "scroll bar"
@@ -289,21 +293,6 @@ rolenames[ROLE_CHECK_MENU_ITEM] = Rolename(
     #
     _("check item"))
 
-rolenames[ROLE_CHECK_MENU] = Rolename(
-    ROLE_CHECK_MENU,
-    # Translators: short braille for the rolename of a check menu (i.e.,
-    # a bizarre construct where the menu item is also a menu).
-    #
-    _("ckm"),
-    # Translators: long braille for the rolename of a check menu (i.e.,
-    # a bizarre construct where the menu item is also a menu).
-    #
-    _("CheckMenu"),
-    # Translators: spoken words for the rolename of a check menu (i.e.,
-    # a bizarre construct where the menu item is also a menu).
-    #
-    _("check menu"))
-
 rolenames[ROLE_COLOR_CHOOSER] = Rolename(
     ROLE_COLOR_CHOOSER,
     # Translators: short braille for the rolename of a color chooser.
@@ -379,8 +368,13 @@ rolenames[ROLE_DESKTOP_FRAME] = Rolename(
 rolenames[ROLE_DIAL] = Rolename(
     ROLE_DIAL,
     # Translators: short braille for the rolename of a dial.
+    # You should attempt to treat it as an abbreviation of
+    # the translated word for "dial".  It is OK to use an
+    # unabbreviated word as long as it is relatively short.
     #
-    _("dial"),
+    # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+    #
+    Q_("shortbraille|dial"),
     # Translators: long braille for the rolename of a dial.
     #
     _("Dial"),
@@ -475,8 +469,13 @@ rolenames[ROLE_FONT_CHOOSER] = Rolename(
 rolenames[ROLE_FORM] = Rolename(
     ROLE_FORM,
     # Translators: short braille for the rolename of a form.
+    # You should attempt to treat it as an abbreviation of
+    # the translated word for "form".  It is OK to use an
+    # unabbreviated word as long as it is relatively short.
     #
-    _("form"),
+    # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+    #
+    Q_("shortbraille|form"),
     # Translators: long braille for the rolename of a form.
     #
     _("Form"),
@@ -784,21 +783,6 @@ rolenames[ROLE_RADIO_MENU_ITEM] = Rolename(
     #
     _("radio menu item"))
 
-rolenames[ROLE_RADIO_MENU] = Rolename(
-    ROLE_RADIO_MENU,
-    # Translators: short braille for the rolename of a radio menu, which
-    # is a bizarre construct where the menu item is also a menu.
-    #
-    _("rdmnu"),
-    # Translators: long braille for the rolename of a radio_menu, which
-    # is a bizarre construct where the menu item is also a menu.
-    #
-    _("RadioMenu"),
-    # Translators: spoken words for the rolename of a radio_menu, which
-    # is a bizarre construct where the menu item is also a menu.
-    #
-    _("radio menu"))
-
 rolenames[ROLE_ROOT_PANE] = Rolename(
     ROLE_ROOT_PANE,
     # Translators: short braille for the rolename of a root pane.
@@ -1185,7 +1169,43 @@ rolenames[ROLE_EMBEDDED] = Rolename(
     #
     _("embedded component"))
 
-def getSpeechForRoleName(obj):
+
+# [[[TODO: eitani - This is for backward compatability, we now put in pyatspi 
+# keys into the dictionary]]]
+
+_legacy_rolename_keys = rolenames.keys()
+
+for sym in dir(pyatspi):
+    if sym.startswith('ROLE_'):
+        possible_key = sym.replace('ROLE_','').replace('_','').lower()
+        for key in _legacy_rolename_keys:
+            if key.replace(' ','') == possible_key:
+                pyatspi_role = getattr(pyatspi, sym)
+                rolenames[pyatspi_role] = rolenames[key]
+
+def _adjustRole(obj, role):
+    """Adjust the role to what the role really is.
+    """
+
+    # Return fake "menu" role names.
+    #
+    if (role == pyatspi.ROLE_MENU_ITEM) \
+          and (obj.childCount > 0):
+        role = ROLE_MENU
+
+    # If this is an ARIA button with the "haspopup:true" attribute, then
+    # it's really a menu.
+    #
+    if role in [pyatspi.ROLE_PUSH_BUTTON, pyatspi.ROLE_MENU_ITEM]:
+        attributes = obj.getAttributes()
+        for attribute in attributes:
+            if attribute.startswith("haspopup:true"):
+                role = ROLE_MENU
+                break
+
+    return role
+
+def getSpeechForRoleName(obj, role=None):
     """Returns the localized name of the given Accessible object; the name is
     suitable to be spoken.  If a localized name cannot be discovered, this
     will return the string as defined by the at-spi.
@@ -1197,18 +1217,22 @@ def getSpeechForRoleName(obj):
     to be spoken.
     """
 
-    name = obj.role
-    if rolenames.has_key(name):
-        return rolenames[name].speech
+    role = _adjustRole(obj, role or obj.getRole())
+
+    # If the enum is not in the dictionary, check by name.
+    role_entry = \
+        rolenames.get(role) or rolenames.get(obj.getRoleName())
+    if role_entry:
+        return role_entry.speech
     else:
-        debug.println(debug.LEVEL_WARNING, "No rolename for %s" % name)
-        localizedRoleName = obj.localizedRoleName
+        debug.println(debug.LEVEL_WARNING, "No rolename for %s" % repr(role))
+        localizedRoleName = obj.getLocalizedRoleName()
         if localizedRoleName and len(localizedRoleName):
             return localizedRoleName
         else:
-            return name
+            return repr(role)
 
-def getShortBrailleForRoleName(obj):
+def getShortBrailleForRoleName(obj, role=None):
     """Returns the localized name of the given Accessible object; the name is
     a short string suitable for a Braille display.  If a localized name cannot
     be discovered, this will return the string as defined by the at-spi.
@@ -1220,18 +1244,22 @@ def getShortBrailleForRoleName(obj):
     suitable for a Braille display.
     """
 
-    name = obj.role
-    if rolenames.has_key(name):
-        return rolenames[name].brailleShort
+    role = _adjustRole(obj, role or obj.getRole())
+
+    # If the enum is not in the dictionary, check by name.
+    role_entry = \
+        rolenames.get(role) or rolenames.get(obj.getRoleName())
+    if role_entry:
+        return role_entry.brailleShort
     else:
-        debug.println(debug.LEVEL_WARNING, "No rolename for %s" % name)
-        localizedRoleName = obj.localizedRoleName
+        debug.println(debug.LEVEL_WARNING, "No rolename for %s" % repr(role))
+        localizedRoleName = obj.getLocalizedRoleName()
         if localizedRoleName and len(localizedRoleName):
             return localizedRoleName
         else:
-            return name
+            return repr(role)
 
-def getLongBrailleForRoleName(obj):
+def getLongBrailleForRoleName(obj, role=None):
     """Returns the localized name of the given Accessible object; the name is
     a long string suitable for a Braille display.  If a localized name cannot
     be discovered, this will return the string as defined by the at-spi.
@@ -1243,18 +1271,22 @@ def getLongBrailleForRoleName(obj):
     a Braille display.
     """
 
-    name = obj.role
-    if rolenames.has_key(name):
-        return rolenames[name].brailleLong
+    role = _adjustRole(obj, role or obj.getRole())
+
+    # If the enum is not in the dictionary, check by name.
+    role_entry = \
+        rolenames.get(role) or rolenames.get(obj.getRoleName())
+    if role_entry:
+        return role_entry.brailleLong
     else:
-        debug.println(debug.LEVEL_WARNING, "No rolename for %s" % name)
-        localizedRoleName = obj.localizedRoleName
+        debug.println(debug.LEVEL_WARNING, "No rolename for %s" % repr(role))
+        localizedRoleName = obj.getLocalizedRoleName()
         if localizedRoleName and len(localizedRoleName):
             return localizedRoleName
         else:
-            return name
+            return repr(role)
 
-def getBrailleForRoleName(obj):
+def getBrailleForRoleName(obj, role=None):
     """Returns the localized name of the given Accessible object; the name is
     a string suitable for a Braille display.  If a localized name cannot
     be discovered, this will return the string as defined by the at-spi.
@@ -1268,6 +1300,6 @@ def getBrailleForRoleName(obj):
     """
 
     if settings.brailleRolenameStyle == settings.BRAILLE_ROLENAME_STYLE_SHORT:
-        return getShortBrailleForRoleName(obj)
+        return getShortBrailleForRoleName(obj, role)
     else:
-        return getLongBrailleForRoleName(obj)
+        return getLongBrailleForRoleName(obj, role)
